#include "bonjourregistrar.h"
#include <QSocketNotifier>

#ifdef USE_ZEROCONF

BonjourRegistrar::BonjourRegistrar(QObject *parent)
	: QObject(parent), dnssref(0), bonjourSocket(0)
{
}

#else

BonjourRegistrar::BonjourRegistrar(QObject *parent)
	: QObject(parent), bonjourSocket(0)
{
}

#endif

BonjourRegistrar::~BonjourRegistrar()
{
#ifdef USE_ZEROCONF
	if (dnssref)
	{
		DNSServiceRefDeallocate(dnssref);
		dnssref = 0;
	}
#endif
}

const BonjourRecord &BonjourRegistrar::registeredRecord() const
{
	return finalRecord;
}

#ifdef USE_ZEROCONF
void BonjourRegistrar::registerService(const BonjourRecord &record, quint16 servicePort)
#else
void BonjourRegistrar::registerService(const BonjourRecord &, quint16)
#endif
{
#ifdef USE_ZEROCONF
	if (dnssref)
	{
		qWarning("Already registered a service");
		return;
	}

	quint16 bigEndianPort = servicePort;
#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
	bigEndianPort = ((servicePort & 0x00ff) << 8) | ((servicePort & 0xff00) >> 8);
#endif
	DNSServiceErrorType err = DNSServiceRegister(&dnssref,
												 0, 0, record.serviceName.toUtf8().constData(),
												 record.registeredType.toUtf8().constData(),
												 record.replyDomain.isEmpty()
													? 0
													: record.replyDomain.toUtf8().constData(),
												 0, bigEndianPort, 0, 0, bonjourRegisterService,
												 this);
	if (err != kDNSServiceErr_NoError)
	{
		emit error(err);
	}
	else
	{
		int sockfd = DNSServiceRefSockFD(dnssref);
		if (sockfd == -1)
		{
			emit error(kDNSServiceErr_Invalid);
		}
		else
		{
			bonjourSocket = new QSocketNotifier(sockfd,
												QSocketNotifier::Read, this);
			connect(bonjourSocket, SIGNAL(activated(int)),
					this, SLOT(bonjourSocketReadyRead()));
		}
	}
#endif
}

void BonjourRegistrar::bonjourSocketReadyRead()
{
#ifdef USE_ZEROCONF
	DNSServiceErrorType err = DNSServiceProcessResult(dnssref);
	if (err != kDNSServiceErr_NoError)
		emit error(err);
#endif
}

#ifdef USE_ZEROCONF
void DNSSD_API BonjourRegistrar::bonjourRegisterService(DNSServiceRef,
														DNSServiceFlags,
														DNSServiceErrorType errorCode,
														const char *name,
														const char *regType,
														const char *domain,
														void *data)
{
	BonjourRegistrar *registrar = static_cast<BonjourRegistrar *>(data);
	if (errorCode != kDNSServiceErr_NoError)
	{
		emit registrar->error(errorCode);
	}
	else
	{
		registrar->finalRecord = BonjourRecord(name, regType, domain);
		emit registrar->serviceRegistered(registrar->finalRecord);
	}
}

#endif
