#include "findcmd.h"
#include "sharedfolders.h"
#include "ftpserver.h"
#include <QTimer>
#include <QTcpSocket>
#include "globals.h"
#include "userspath.h"
#include "delay.h"

FINDcmd::FINDcmd(const UserID &uid, const QString &request, ServerThread *parent) : QObject(parent), uid(uid)
{
	this->parent()->send("150 Opening BINARY mode data connection for FIND " + request);

	connect(this->parent()->passv,SIGNAL(error(QAbstractSocket::SocketError)),this,SLOT(finish()));
	connect(this->parent()->passv,SIGNAL(disconnected()),this,SLOT(finish()));

	workQueue.enqueue(QString());

	dead = false;
	req = request;
	k = 0;
	QTimer *timer = new QTimer(this);
	timer->setInterval(10);
	timer->setSingleShot(false);
	connect(timer,SIGNAL(timeout()),this,SLOT(process()));
	timer->start();
}

FINDcmd::~FINDcmd()
{
	disconnect(this);
	dead = true;
}

ServerThread *FINDcmd::parent() const
{
	return static_cast<ServerThread*>(QObject::parent());
}

void FINDcmd::finish()
{
	dead = true;
	disconnect(this);
	parent()->killDTP();
	parent()->send("226 Transfer complete");

	emit activity();
	emit finished();
	deleteLater();
}

void FINDcmd::process()
{
	if (dead)
		return;
	if (workQueue.isEmpty() || parent()->mode != ServerThread::FIND)
	{
		finish();
		return;
	}

	const QRegExp test = QRegExp(req, Qt::CaseInsensitive, QRegExp::Wildcard);

	const QString cur = workQueue.dequeue();

	QStringList entries = UsersPath::listDirs(uid, cur);

	foreach(QString entry, entries)
	{
		if (entry == "." || entry == "..")
			continue;
		workQueue.enqueue(cur + "/" + entry);
	}

	entries = UsersPath::listAll(uid, cur);
	foreach(QString entry, entries)
	{
		if (dead)	return;
		if (entry == "." || entry == "..")
			continue;
		if (!test.exactMatch(entry))
			continue;

		QByteArray data = (cur + Path::separator + entry).toUtf8() + "\r\n";

		int written = 0;
		while (written != -1
			   && written < data.size()
			   && parent()->mode == ServerThread::FIND
			   && parent()->passv->isValid()
			   && parent()->passv->isOpen())
		{
			data.remove(0, written);
			written = parent()->passv->write(data);
			if (dead)	return;
			while (parent()->passv->isValid()
				&& parent()->mode == parent()->FIND
				&& parent()->passv->isOpen()
				&& parent()->passv->bytesToWrite() > 0)
			{
				parent()->passv->waitForBytesWritten(10);
				if (dead)	return;
			}
			FtpServer::pInstance->statsSend(data.size());
			if (k > 100)
			{
				emit activity();
				k = 0;
			}
			++k;
		}
		if (parent()->mode != ServerThread::FIND)
			break;
	}
}
