#include "globals.h"
#if QT_VERSION >= 0x040600
#include <QProcessEnvironment>
#else
#include <QProcess>
#endif

#ifdef Q_OS_WIN32
#include <windows.h>
#include <lmcons.h>
#include <QSysInfo>
#endif

namespace qshare
{
	QString getLocalUserName();

	QSettings *settings = NULL;
	QTranslator *qAppTranslator = NULL;
	QTranslator *qQtTranslator = NULL;
	const QString localUserName = getLocalUserName();
	bool bGUImode = false;

	QString getLocalUserName()
	{
#ifdef Q_OS_LINUX
#if QT_VERSION >= 0x040600
		return QProcessEnvironment::systemEnvironment().value("USER");
#else
		QStringList env = QProcess::systemEnvironment().filter(QRegExp("USER=*"));
		return env.empty() ? QString("user") : env.front().remove(0, 5);
#endif
#elif defined(Q_OS_WIN32)
		QString userName;

#ifdef UNICODE
		if (QSysInfo::windowsVersion() & QSysInfo::WV_NT_based)
		{
			TCHAR winUserName[UNLEN + 1]; // UNLEN is defined in LMCONS.H
			DWORD winUserNameSize = sizeof(winUserName);
			GetUserName( winUserName, &winUserNameSize );
			userName = QString::fromWCharArray(winUserName);
		} else
#endif
		{
			char winUserName[UNLEN + 1]; // UNLEN is defined in LMCONS.H
			DWORD winUserNameSize = sizeof(winUserName);
			GetUserNameA( winUserName, &winUserNameSize );
			userName = QString::fromLocal8Bit( winUserName );
		}
		return userName;
#else
		return "user";
#endif
	}
}
