#ifndef IPFILESERVER_H
#define IPFILESERVER_H

#include "ipobject.h"
#include <QHash>
#include <QFile>
#include "globals.h"

class UserID;

class IPFileServer : public IPObject
{
public:
    IPFileServer();
	~IPFileServer();

	bool seek(const QString &user, quint32 ID, qint64 offset);
	bool isReadable(const QString &user, quint32 ID);
	bool isWritable(const QString &user, quint32 ID);
	quint32 open(const UserID &uid, const QString &user, const QString &filename, QFile::OpenMode mode);
	void close(const QString &user, quint32 ID);
	void flush(const QString &user, quint32 ID);
	QByteArray read(const QString &user, quint32 ID, qint64 maxlen);
	void write(const QString &user, quint32 ID, const QByteArray &data);

protected:
	virtual void receive(const QByteArray &message);

	QByteArray waitForID(const quint32 ID);

	template<class T>
		QByteArray remote(const QString &type, const T &arg0)
		{
			const quint32 ID = nextTransactionID++;
			send(type, qshare::localUserName, ID, arg0);
			return waitForID(ID);
		}
	template<class T, class V>
		QByteArray remote(const QString &type, const T &arg0, const V &arg1)
		{
			const quint32 ID = nextTransactionID++;
			send(type, qshare::localUserName, ID, arg0, arg1);
			return waitForID(ID);
		}
	template<class T, class V, class U>
		QByteArray remote(const QString &type, const T &arg0, const V &arg1, const U &arg2)
		{
			const quint32 ID = nextTransactionID++;
			send(type, qshare::localUserName, ID, arg0, arg1, arg2);
			return waitForID(ID);
		}
	template<class T, class V, class U, class W>
		QByteArray remote(const QString &type, const T &arg0, const V &arg1, const U &arg2, const W &arg3)
		{
			const quint32 ID = nextTransactionID++;
			send(type, qshare::localUserName, ID, arg0, arg1, arg2, arg3);
			return waitForID(ID);
		}

private:
	quint32 nextTransactionID;
	quint32 fileID;
	QHash<quint32, QByteArray> messageQueue;
	QHash<quint32, QFile*> files;

private:
	static IPFileServer *pInstance;
public:
	static IPFileServer *instance();
};

#endif // IPFILESERVER_H
