#ifndef IPOBJECT_H
#define IPOBJECT_H

#include <QObject>
#include <QIODevice>
#include <QMutex>

class QTcpSocket;
class QTcpServer;
#ifndef QT_NO_SYSTEMSEMAPHORE
class QSystemSemaphore;
#endif

class IPObject : public QObject
{
    Q_OBJECT
public:
	explicit IPObject(const QString &name, const quint16 port, QObject *parent = 0);
	virtual ~IPObject();

	bool isServing() const;
	void waitForData(int msec);

protected:
	virtual void receive(const QByteArray &message) = 0;
	void send(const QByteArray &message);
	void send(const QString &message);
	void send(const char *message);
	template<class T>
		void send(const QString &message, const T &arg0);
	template<class T, class U>
		void send(const QString &message, const T &arg0, const U &arg1);
	template<class T, class U, class V>
		void send(const QString &message, const T &arg0, const U &arg1, const V &arg2);
	template<class T, class U, class V, class W>
		void send(const QString &message, const T &arg0, const U &arg1, const V &arg2, const W &arg3);
	template<class T, class U, class V, class W, class X>
		void send(const QString &message, const T &arg0, const U &arg1, const V &arg2, const W &arg3, const X &arg4);
	template<class T, class U, class V, class W, class X, class Y>
		void send(const QString &message, const T &arg0, const U &arg1, const V &arg2, const W &arg3, const X &arg4, const Y &arg5);

signals:
	void enterServerMode();
	void enterClientMode();
	void clientLeft();

public slots:
	void link();

private slots:
	void processConnection();
	void broadcast();
	void receive();
	void handleDisconnection();

private:
	const quint16 port;
	QTcpServer *server;
	QTcpSocket *client;
#ifndef QT_NO_SYSTEMSEMAPHORE
	QSystemSemaphore *lock;
#endif
	QMutex mutex;
	bool bLinkInProgress;
};

template<class T>
	void IPObject::send(const QString &message, const T &arg0)
	{
		QByteArray data;
		QDataStream stream(&data, QIODevice::WriteOnly);
		stream << message << arg0;
		send(data);
	}

template<class T, class U>
	void IPObject::send(const QString &message, const T &arg0, const U &arg1)
	{
		QByteArray data;
		QDataStream stream(&data, QIODevice::WriteOnly);
		stream << message << arg0 << arg1;
		send(data);
	}

template<class T, class U, class V>
	void IPObject::send(const QString &message, const T &arg0, const U &arg1, const V &arg2)
	{
		QByteArray data;
		QDataStream stream(&data, QIODevice::WriteOnly);
		stream << message << arg0 << arg1 << arg2;
		send(data);
	}

template<class T, class U, class V, class W>
	void IPObject::send(const QString &message, const T &arg0, const U &arg1, const V &arg2, const W &arg3)
	{
		QByteArray data;
		QDataStream stream(&data, QIODevice::WriteOnly);
		stream << message << arg0 << arg1 << arg2 << arg3;
		send(data);
	}

template<class T, class U, class V, class W, class X>
	void IPObject::send(const QString &message, const T &arg0, const U &arg1, const V &arg2, const W &arg3, const X &arg4)
	{
		QByteArray data;
		QDataStream stream(&data, QIODevice::WriteOnly);
		stream << message << arg0 << arg1 << arg2 << arg3 << arg4;
		send(data);
	}

template<class T, class U, class V, class W, class X, class Y>
	void IPObject::send(const QString &message, const T &arg0, const U &arg1, const V &arg2, const W &arg3, const X &arg4, const Y &arg5)
	{
		QByteArray data;
		QDataStream stream(&data, QIODevice::WriteOnly);
		stream << message << arg0 << arg1 << arg2 << arg3 << arg4 << arg5;
		send(data);
	}

#endif // IPOBJECT_H
