#include <QtGui/QApplication>
#include <QTranslator>
#include <QLocale>
#include <QSettings>
#include <QSplashScreen>
#include <QTimer>
#include "mainwindow.h"
#include "globals.h"
#include "netdiscovery.h"
#include "ftpserver.h"
#include <iostream>
#include <QTextStream>
#include <QDebug>
#include "ipfileserver.h"
#include "sharedpath.h"
#include "config.h"

using namespace qshare;


int main(int argc, char *argv[])
{
	QStringList arguments;
	for(int i = 0 ; i < argc ; ++i)
		arguments << argv[i];

	const bool bServerOnly = arguments.contains("--server-only");
	const bool bHelp = arguments.contains("--help");

	bGUImode = !bServerOnly;

	QApplication a(argc, argv, bGUImode);
	QApplication::setApplicationName(APP_NAME);
	QApplication::setOrganizationName(ORG_NAME);
	QApplication::setApplicationVersion(APP_VERSION);

	settings = new QSettings;

	if (arguments.contains("--enable-zeroconf"))
		Config::setZeroconfEnabled(true);
	if (arguments.contains("--disable-zeroconf"))
		Config::setZeroconfEnabled(false);

	//***************************************************************
	//*                                                             *
	//*                Display command line options                 *
	//*                                                             *
	//***************************************************************
	if (bHelp)
	{
		QString buf;
		QTextStream out(&buf);
		out << APP_NAME << ' ' << APP_VERSION << " by " << ORG_NAME << '\n'
			<< '\n'
			<< "syntax:\n"
			<< argv[0] << " [options]\n"
			<< '\n'
			<< "options:\n"
			<< "--help                " << '\t' << "Display this screen\n"
			<< "--server-only         " << '\t' << "Run without GUI\n"
			<< "--share=path          " << '\t' << "Add a shared folder\n"
			<< "--unshare=path        " << '\t' << "Remove a shared folder\n"
			<< "--disable-zeroconf    " << '\t' << "Disable Zeroconf\n"
			<< "--enable-zeroconf     " << '\t' << "Enable Zeroconf\n"
			<< "\noptions controlling GUI:\n"
			<< "MainWindow::show()    " << '\t' << "Show main window\n"
			<< "About::show()         " << '\t' << "Show the about dialog\n"
			<< "Config::show()        " << '\t' << "Show Config dialog\n"
			<< "Stats::show()         " << '\t' << "Show statistics dialog\n"
			<< "SearchFile::show()    " << '\t' << "Show the search dialog\n"
			<< "MainWindow::hide()    " << '\t' << "Hide main window\n"
			<< "About::hide()         " << '\t' << "Hide about dialog\n"
			<< "Config::hide()        " << '\t' << "Hide config dialog\n"
			<< "Stats::hide()         " << '\t' << "Hide statistics dialog\n"
			<< "SearchFile::hide()    " << '\t' << "Hide search dialog\n"
			<< "MainWindow::exit()    " << '\t' << "terminate the running instance\n"
			<< "exit                  " << '\t' << "terminate the running instance\n";
		std::cout << buf.toAscii().data() << std::endl;
		return 0;
	}

	//***************************************************************
	//*                                                             *
	//*                  Server only mode (no GUI)                  *
	//*                                                             *
	//***************************************************************
	if (bServerOnly)
	{
		QObject cleaner;
		NetDiscovery::instance()->setParent(&cleaner);
		SharedPath::instance()->setParent(&cleaner);
		IPFileServer::instance()->setParent(&cleaner);
		FtpServer::startServer();
		return a.exec();
	}

	// Set up the splash screen if needed
	QTimer *timer = NULL;
	if (argc == 1)
	{
		settings->beginGroup("config");
		if (settings->value("splash", true).toBool())
		{
			QSplashScreen *splash = new QSplashScreen(QPixmap(":/images/splash.jpg"), Qt::WindowStaysOnTopHint);
			timer = new QTimer;
			timer->setSingleShot(true);
			timer->setInterval(1000);
			QTimer::connect(timer, SIGNAL(timeout()), splash, SLOT(deleteLater()));
			QTimer::connect(timer, SIGNAL(timeout()), timer, SLOT(deleteLater()));
			splash->show();
		}
		settings->endGroup();
	}

	//***************************************************************
	//*                                                             *
	//*                            GUI mode                         *
	//*                                                             *
	//***************************************************************
	qAppTranslator = new QTranslator;
	qQtTranslator = new QTranslator;
	bool showMainWindow = false;

	QString localeSuffix = settings->value("locale", QLocale::system().name().left(2)).toString() + ".qm";

	if (!qAppTranslator->load(QString(I18N_PATH) + "qshare_" + localeSuffix))
	    qDebug() << "App Translator" << (QString(I18N_PATH) + "qshare_" + localeSuffix) << "cannot be loaded";

#ifdef SYSTEM_I18N_PATH
	if (!qQtTranslator->load(QString(SYSTEM_I18N_PATH) + "qt_" + localeSuffix))
		qDebug() << "Qt Translator" << (QString(SYSTEM_I18N_PATH) + "qt_" + localeSuffix) << "cannot be loaded";
#else
	if (!qQtTranslator->load(QString(I18N_PATH) + "qt_" + localeSuffix))
	    qDebug() << "Qt Translator" << (QString(I18N_PATH) + "qt_" + localeSuffix) << "cannot be loaded";
#endif

	showMainWindow = settings->value("firstStart", true).toBool();
	settings->setValue("firstStart", false);

	a.installTranslator(qAppTranslator);
	a.installTranslator(qQtTranslator);

	a.setQuitOnLastWindowClosed(false);
    MainWindow w;
	settings->setParent(&w);
	if (showMainWindow)
		w.show();

	if (timer)
		timer->start();

	return a.exec();
}
