#include "searchthread.h"
#include "netdiscovery.h"
#include "searchfile.h"
#include <QBuffer>
#include <QDebug>
#include <QTcpSocket>
#include <QHostAddress>

SearchThread::SearchThread(const QString &computer, const QString &request, QObject *parent) : QObject(parent)
{
	this->computer = computer;
	this->request = request;

	sock = new QTcpSocket(this);
	connect(sock,SIGNAL(error(QAbstractSocket::SocketError)),this,SLOT(stopSearch()));
}

void SearchThread::start()
{
	connect(sock,SIGNAL(connected()),this,SLOT(connected()));
	connect(sock,SIGNAL(readyRead()),this,SLOT(processReply()));

	Computer server = NetDiscovery::instance()->getComputer(computer);
	sock->connectToHost(server.getHostname(), server.getPort());
}

void SearchThread::connected()
{
	sock->write("USER anonymous\r\n");
	sock->write("PASV\r\n");
	sock->flush();
}

void SearchThread::processReply()
{
	while(sock->canReadLine())
	{
		QByteArray line = sock->readLine(65536);
		int code = line.left(3).toInt();
		QByteArray details = line.right(line.size() - 3).trimmed();

		if (code == 227)		// Enter passive mode
		{
			QList<QByteArray> args = line.split(',');
			QList<QByteArray> last = args.last().split(')');
			int port = 256 * args.at(args.size() - 2).toInt() + last.first().toInt();

			dtp = new QTcpSocket(this);
			connect(dtp,SIGNAL(readyRead()),this,SLOT(readResults()));
			dtp->connectToHost(sock->peerAddress(), port);
			sock->write(("FIND " + request + "\r\n").toUtf8());
		}
		else if (code == 226)		// We're done
		{
			stopSearch();
		}
		else if (code / 100 > 2)	// There is an error
		{
			stopSearch();
		}
	}
}

void SearchThread::readResults()
{
	while (dtp->canReadLine())
	{
		QByteArray line = dtp->readLine(65536).trimmed();
		emit found(computer, QString::fromUtf8(line.data(), line.size()));
	}
}

void SearchThread::stopSearch()
{
	if (sock->isOpen())
		sock->close();
	deleteLater();
}
