#ifndef SERVERTHREAD_H
#define SERVERTHREAD_H

#include <QFile>
#include "path.h"
#include "userid.h"

class QTcpSocket;
class QTcpServer;

#define DTP_BUFFER_SIZE	0x100000

class ServerThread : public QObject
{
	Q_OBJECT

	friend class STORcmd;
	friend class FINDcmd;
	friend class RETRcmd;
	friend class LISTcmd;

public:
	enum DTP_MODE { NONE, RETR, STOR, APPE, FIND };
public:
	ServerThread(QTcpSocket *s, QObject *parent);
	~ServerThread();

	bool isConnected();
	DTP_MODE getMode();
	QString getLogin();
	QString getFilename();
	QString getAddress();
	bool UTF8mode()	const {	return utf8Enabled;	}

public slots:
	void processCommand();
	void connection();
	void pasvConnection();
	void portConnection();
	void killDTP();
	void errorDTP();
	void start();
	void activate();
	void list();
	void nlst();

signals:
	void activity();
	void DTPconnected();

protected:
	void send(const QString &str);
	void openPasv();
	void openEpsv();
	QString toStringGuessEncoding(const QByteArray &data);

private:
	bool waitingForDTP;
	QString cmdParam;
	QTcpSocket *sock;
	QTcpSocket *passv;
	QTcpServer *serv;
	QFile file;
	QString login;
	QString renameFrom;
	qint64 skip;
	bool logged;
	bool login_accepted;
	bool pass_accepted;
	Path path;
	QString filename;
	DTP_MODE mode;
	bool utf8Enabled;
	UserID uid;
};

#endif // SERVERTHREAD_H
