#ifndef SHAREDPATH_H
#define SHAREDPATH_H

#include "ipobject.h"
#include <QHash>
#include <QSet>
#include "globals.h"
#include <QDateTime>

class UserID;

class SharedPath : public IPObject
{
private:
    SharedPath();
	~SharedPath();

protected:
	virtual void receive(const QByteArray &message);

	QByteArray waitForID(const quint32 ID);

	template<class T>
		QByteArray remote(const QString &type, const T &arg0)
		{
			const quint32 ID = nextTransactionID++;
			send(type, qshare::localUserName, ID, arg0);
			return waitForID(ID);
		}
	template<class T, class V>
		QByteArray remote(const QString &type, const T &arg0, const V &arg1)
		{
			const quint32 ID = nextTransactionID++;
			send(type, qshare::localUserName, ID, arg0, arg1);
			return waitForID(ID);
		}
	template<class T, class V, class U>
		QByteArray remote(const QString &type, const T &arg0, const V &arg1, const U &arg2)
		{
			const quint32 ID = nextTransactionID++;
			send(type, qshare::localUserName, ID, arg0, arg1, arg2);
			return waitForID(ID);
		}
	template<class T, class V, class U, class W>
		QByteArray remote(const QString &type, const T &arg0, const V &arg1, const U &arg2, const W &arg3)
		{
			const quint32 ID = nextTransactionID++;
			send(type, qshare::localUserName, ID, arg0, arg1, arg2, arg3);
			return waitForID(ID);
		}

private:
	static SharedPath *pInstance;
	static const QChar separator;
public:
	static SharedPath *instance();

public:
	bool mkd(const UserID &uid, const QString &user, const QString &name);
	bool rmd(const UserID &uid, const QString &user, const QString &name);
	bool dele(const UserID &uid, const QString &user, const QString &name);
	bool rename(const UserID &uid, const QString &user, const QString &from, const QString &to);
	void list(const UserID &uid, const QString &user, QList<QByteArray> &out, const QString &name, bool utf8 = false);
	void nlst(const UserID &uid, const QString &user, QList<QByteArray> &out, const QString &name, bool utf8 = false);
	bool exists(const UserID &uid, const QString &user, const QString &name);
	bool isDir(const UserID &uid, const QString &user, const QString &name);
	bool isFile(const UserID &uid, const QString &user, const QString &name);
	void chmod(const UserID &uid, const QString &user, const QString &name, QByteArray perms);
	qint64 size(const UserID &uid, const QString &user, const QString &name);
	QDateTime lastModified(const UserID &uid, const QString &user, const QString &name);
	bool isReadable(const UserID &uid, const QString &user, const QString &name);
	bool isWritable(const UserID &uid, const QString &user, const QString &name);
	QStringList listDirs(const UserID &uid, const QString &user, const QString &name);
	QStringList listAll(const UserID &uid, const QString &user, const QString &name);

private:
	quint32 nextTransactionID;
	QHash<quint32, QByteArray> messageQueue;
};

#endif // SHAREDPATH_H
