#include "trayicon.h"
#include <QtGui/QApplication>
#include "config.h"
#include "netdiscovery.h"
#include "ftpserver.h"
#include "stats.h"
#include "searchfile.h"
#include "about.h"
#include <QTimer>

TrayIcon::TrayIcon(QObject *parent) : QSystemTrayIcon(parent), QMutex(QMutex::Recursive)
{
	rebuildGUI();
	setVisible(true);

	QTimer *timer = new QTimer(this);
	timer->setInterval(1000);
	timer->setSingleShot(true);
	connect(timer,SIGNAL(timeout()),this,SLOT(showStartupMessage()));
	connect(this,SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this,SLOT(activating(QSystemTrayIcon::ActivationReason)));
	connect(NetDiscovery::instance(), SIGNAL(serverListChanged()), this, SLOT(refreshComputers()));
	timer->start();
}

void TrayIcon::showStartupMessage()
{
	showMessage(tr("qShare started"), tr("qShare is up and running"), Information, 1000);
}

void TrayIcon::showSearchFileDialog()
{
	SearchFile::instance()->show();
}

void TrayIcon::showStatsDialog()
{
	Stats::instance()->show();
}

void TrayIcon::rebuildGUI()
{
	QMutexLocker mLock(this);

	QMenu *mTray = new QMenu;
	mComputers = new QMenu(tr("computers"));

	mTray->addAction(QIcon(":/icons/information.png"), tr("&About"), About::instance(), SLOT(show()));
	mTray->addAction(QIcon(":/icons/visible-on.png"), tr("&Show"), parent(), SLOT(show()));
	mTray->addAction(QIcon(":/icons/options.png"), tr("&Config"), Config::instance(), SLOT(show()));
	mTray->addAction(QIcon(":/icons/chart.png"), tr("&Stats"), this, SLOT(showStatsDialog()));
	mTray->addAction(QIcon(":/icons/find.png"), tr("&Find"), this, SLOT(showSearchFileDialog()));
	mTray->addMenu(mComputers);
	mTray->addSeparator();
	mTray->addAction(QIcon(":/icons/exit.png"),tr("&Exit"), qApp, SLOT(quit()));

	refreshComputers();

	setContextMenu(mTray);
	setIcon(QIcon(":/icons/tray.png"));
	setToolTip(tr("qShare"));
}

bool TrayIcon::event(QEvent *event)
{
	switch(event->type())
	{
	case QEvent::LanguageChange:
		rebuildGUI();
		break;
	default:
		break;
	};
	return QSystemTrayIcon::event(event);
}

void TrayIcon::refreshComputers()
{
	QMutexLocker mLock(this);

	mComputers->clear();
	computers = NetDiscovery::instance()->getServerList();

	QString names;
	for(QList<Computer>::iterator i = computers.begin() ; i != computers.end() ; ++i)
	{
		mComputers->addAction(QIcon(":/icons/computer.png"), i->getName(), &(*i), SLOT(open()));
		if (!names.isEmpty())
			names += '\n';
        names += '\t' + i->getName();
	}
	mComputers->addAction(QIcon(":/icons/refresh.png"),tr("refresh list"),NetDiscovery::instance(),SLOT(forceRefresh()));
	if (names.isEmpty())
        setToolTip(tr("qShare") + ' ' + tr("on") + ' ' + Config::instance()->getHostname());
	else
        setToolTip(tr("qShare") + ' ' + tr("on") + ' ' + Config::instance()->getHostname() + '\n' + names);
}

void TrayIcon::activating(QSystemTrayIcon::ActivationReason reason)
{
	switch(reason)
	{
	case Trigger:
		dynamic_cast<QWidget*>(parent())->setVisible(!dynamic_cast<QWidget*>(parent())->isVisible());
		break;
	case Context:
		break;
	default:
		break;
	};
}
