#include "userspath.h"
#include "ftpserver.h"
#include <QStringList>
#include "sharedpath.h"

const QChar UsersPath::separator('/');

bool UsersPath::mkd(const UserID &uid, const QString &name)
{
#define HEAD(RET)\
	QStringList path = name.split(separator, QString::SkipEmptyParts);\
	if (path.isEmpty())\
		return RET;\
	const QSet<QString> &users = FtpServer::instance()->getUsers();\
	if (!users.contains(path[0]))\
		return RET

	HEAD(false);

	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->mkd(uid, user, makePath(path));
}

bool UsersPath::rmd(const UserID &uid, const QString &name)
{
	HEAD(false);

	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->rmd(uid, user, makePath(path));
}

bool UsersPath::dele(const UserID &uid, const QString &name)
{
	HEAD(false);

	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->dele(uid, user, makePath(path));
}

bool UsersPath::rename(const UserID &uid, const QString &from, const QString &to)
{
	QStringList pathFrom = from.split(separator, QString::SkipEmptyParts);
	QStringList pathTo = to.split(separator, QString::SkipEmptyParts);
	const QSet<QString> &users = FtpServer::instance()->getUsers();

	if (pathFrom.isEmpty() || pathTo.isEmpty())
		return false;

	if (pathFrom[0] != pathTo[0])
		return false;

	if (!users.contains(pathFrom[0]))
		return false;

	const QString user = pathFrom[0];
	pathFrom.removeFirst();
	pathTo.removeFirst();
	return SharedPath::instance()->rename(uid, user, makePath(pathFrom), makePath(pathTo));
}

void UsersPath::list(const UserID &uid, QList<QByteArray> &out, const QString &name, bool utf8)
{
	QStringList path = name.split(separator, QString::SkipEmptyParts);
	const QSet<QString> &users = FtpServer::instance()->getUsers();
	if (path.isEmpty())
	{
		const int maxSizeLength(1);
		int maxGroupLength(0);
		foreach(QString user, users)
			maxGroupLength = qMax(maxGroupLength, user.length());
		int maxOwnerLength(maxGroupLength);
		foreach(QString user, users)
		{
			QByteArray line;
			line += "dr-xr-xr-x   1 ";
			line += user;
			line += " ";
			line += QString(maxOwnerLength - user.length(), ' ');
			line += user;
			line += QString(maxGroupLength + maxSizeLength - user.length(), ' ');
			line += "1 Jan   01 00:00 ";
			if (utf8)
				line += user.toUtf8();
			else
				line += user.toAscii();
			out << line;
		}
		return;
	}
	if (!users.contains(path[0]))
		return;

	const QString user = path[0];
	path.removeFirst();
	SharedPath::instance()->list(uid, user, out, makePath(path), utf8);
}

void UsersPath::nlst(const UserID &uid, QList<QByteArray> &out, const QString &name, bool utf8)
{
	QStringList path = name.split(separator, QString::SkipEmptyParts);
	const QSet<QString> &users = FtpServer::instance()->getUsers();
	if (path.isEmpty())
	{
		foreach(QString user, users)
			out << (utf8 ? user.toUtf8() : user.toAscii());
		return;
	}
	if (!users.contains(path[0]))
		return;

	const QString user = path[0];
	path.removeFirst();
	SharedPath::instance()->nlst(uid, user, out, makePath(path), utf8);
}

bool UsersPath::exists(const UserID &uid, const QString &name)
{
	if (name == "/")	return true;
	HEAD(false);
	if (path.size() == 1)
		return true;

	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->exists(uid, user, makePath(path));
}

bool UsersPath::isDir(const UserID &uid, const QString &name)
{
	if (name == "/")
		return true;
	HEAD(false);
	if (path.size() == 1)
		return true;

	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->isDir(uid, user, makePath(path));
}

bool UsersPath::isFile(const UserID &uid, const QString &name)
{
	HEAD(false);
	if (path.size() == 1)
		return false;

	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->isFile(uid, user, makePath(path));
}

void UsersPath::chmod(const UserID &uid, const QString &name, QByteArray perms)
{
	HEAD();
	if (path.size() == 1)
		return;

	const QString user = path[0];
	path.removeFirst();
	SharedPath::instance()->chmod(uid, user, makePath(path), perms);
}

QString UsersPath::makePath(const QStringList &keys)
{
	return separator + keys.join(separator);
}

qint64 UsersPath::size(const UserID &uid, const QString &name)
{
	HEAD(0);
	if (path.size() == 1)
		return 0;

	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->size(uid, user, makePath(path));
}

QDateTime UsersPath::lastModified(const UserID &uid, const QString &name)
{
	HEAD(QDateTime());
	if (path.size() == 1)
		return QDateTime();

	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->lastModified(uid, user, makePath(path));
}

bool UsersPath::isReadable(const UserID &uid, const QString &name)
{
	HEAD(false);
	if (path.size() == 1)
		return false;

	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->isReadable(uid, user, makePath(path));

}

bool UsersPath::isWritable(const UserID &uid, const QString &name)
{
	HEAD(false);
	if (path.size() == 1)
		return false;

	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->isWritable(uid, user, makePath(path));

}

QStringList UsersPath::listDirs(const UserID &uid, const QString &name)
{
	QStringList path = name.split(separator, QString::SkipEmptyParts);
	if (path.isEmpty())
		return QStringList::fromSet(FtpServer::instance()->getUsers());

	const QSet<QString> &users = FtpServer::instance()->getUsers();
	if (!users.contains(path[0]))
		return QStringList();
	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->listDirs(uid, user, makePath(path));
}

QStringList UsersPath::listAll(const UserID &uid, const QString &name)
{
	QStringList path = name.split(separator, QString::SkipEmptyParts);
	if (path.isEmpty())
		return QStringList::fromSet(FtpServer::instance()->getUsers());

	const QSet<QString> &users = FtpServer::instance()->getUsers();
	if (!users.contains(path[0]))
		return QStringList();
	const QString user = path[0];
	path.removeFirst();
	return SharedPath::instance()->listAll(uid, user, makePath(path));
}
